/*---------------------------------------------------------------------------
 
  FILENAME:
        http.c
 
  PURPOSE:
        Provide the wview http packet generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        07/12/2005      M.S. Teel       0               
        12/09/2007      M.S. Teel       1               Add WeatherForYou.com
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>
#include <radprocess.h>
#include <radmsgRouter.h>

/*  ... Local include files
*/
#include <http.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEW_HTTPD_WORK         httpWork;

static char                     httpBuffer[1024];


enum ConfigIds
{
    CFG_ID_STATIONID            = 0,
    CFG_ID_PASSWORD             = 1,
    CFG_ID_YOUSTATIONID         = 2,
    CFG_ID_YOUPASSWORD          = 3
};

static char *configIDs[] =
{
    "STATIONID",
    "PASSWORD",
    "YOUSTATIONID",
    "YOUPASSWORD"
};

/* ... methods
*/
// curl recv callback
static size_t recv_data (void *buffer, size_t size, size_t nmemb, void *userp)
{
    // we don't care about the recvd data
    return size*nmemb;
}

static void processWUNDERGROUND (WVIEW_MSG_ARCHIVE_NOTIFY *notify)
{
    RADSOCK_ID          socket;
    time_t              ntime;
    struct tm           gmTime;
    int                 length = 0;
    char                *serv;
    int                 port;
    char                version[64];
    CURL                *curl;
    CURLcode            res;
    char                curlError[CURL_ERROR_SIZE];
    char                tempBfr[194];
    float               rainIN = sensorAccumGetTotal (httpWork.rainAccumulator);
    
    // format the WUNDERGROUND data
    ntime = time (NULL);
    gmtime_r (&ntime, &gmTime);
    length = sprintf (httpBuffer, "http://weatherstation.wunderground.com/weatherstation/updateweatherstation.php?");
    length += sprintf (&httpBuffer[length], "ID=%s&PASSWORD=%s",
                       httpWork.stationId, httpWork.password);

    length += sprintf (&httpBuffer[length], "&dateutc=%4.4d-%2.2d-%2.2d+%2.2d%%3a%2.2d%%3a%2.2d",
                       gmTime.tm_year + 1900, gmTime.tm_mon + 1, gmTime.tm_mday, 
                       gmTime.tm_hour, gmTime.tm_min, gmTime.tm_sec);

    // check for any wind registered
    if (notify->wspeed >= 0 || notify->hiwspeed >= 0)
    {
        length += sprintf (&httpBuffer[length], "&winddir=%3.3d", notify->winddir);
    }
    
    length += sprintf (&httpBuffer[length], "&windspeedmph=%3.3d", 
                       (notify->wspeed >= 0) ? notify->wspeed : 0);
    length += sprintf (&httpBuffer[length], "&windgustmph=%3.3d", 
                       (notify->hiwspeed >= 0) ? notify->hiwspeed : 0);

    if (notify->humidity >= 0 && notify->humidity <= 100)
    {
        length += sprintf (&httpBuffer[length], "&humidity=%d", notify->humidity);
    }

    length += sprintf (&httpBuffer[length], "&tempf=%3.3d.%1.1d", 
                       notify->temp/10, notify->temp%10);
    
    length += sprintf (&httpBuffer[length], "&rainin=%.2f", rainIN);
    
    length += sprintf (&httpBuffer[length], "&baromin=%2.2d.%2.2d", 
                       notify->barom/1000, (notify->barom%1000)/10);

    length += sprintf (&httpBuffer[length], "&dewptf=%2.2d.%3.3d", 
                       notify->dewpoint/10, (notify->dewpoint%10)*100);

    strcpy (version, globalWviewVersionStr);
    version[5] = '-';
    length += sprintf (&httpBuffer[length], "&weather=&clouds=&softwaretype=%s&action=updateraw",
                       version);


    strncpy (tempBfr, httpBuffer, 192);
    wvutilsLogEvent (PRI_STATUS, "WUNDERGROUND-send: %s", tempBfr);
    if (strlen(httpBuffer) > 192)
    {
        strncpy (tempBfr, &httpBuffer[192], 192);
        wvutilsLogEvent (PRI_STATUS, "WUNDERGROUND-send: %s", tempBfr);
    }


    // OK, now use libcurl to execute the HTTP "GET"
    curl = curl_easy_init ();
    if (curl == NULL)
    {
        radMsgLog (PRI_HIGH, "WUNDERGROUND-error: failed to initialize curl!");
        return;
    }
    
    curl_easy_setopt (curl, CURLOPT_URL, httpBuffer);
    curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, recv_data);
    curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, curlError);
    curl_easy_setopt (curl, CURLOPT_TIMEOUT, 60);

    res = curl_easy_perform (curl);
    if (res != 0)
    {
        radMsgLog (PRI_HIGH, "WUNDERGROUND-error: %s", curlError);
    }
    
    curl_easy_cleanup (curl);
    return;
}

static void processWEATHERFORYOU (WVIEW_MSG_ARCHIVE_NOTIFY *notify)
{
    RADSOCK_ID          socket;
    time_t              ntime;
    struct tm           gmTime;
    int                 length = 0;
    char                *serv;
    int                 port;
    char                version[64];
    CURL                *curl;
    CURLcode            res;
    char                curlError[CURL_ERROR_SIZE];
    char                tempBfr[194];
    float               rainIN = sensorAccumGetTotal (httpWork.rainAccumulator);
    
    // format the WEATHERFORYOU data
    ntime = time (NULL);
    gmtime_r (&ntime, &gmTime);
    length = sprintf (httpBuffer, "http://www.hamweather.net/weatherstations/pwsupdate.php?");
    length += sprintf (&httpBuffer[length], "ID=%s&PASSWORD=%s",
                       httpWork.youstationId, httpWork.youpassword);

    length += sprintf (&httpBuffer[length], "&dateutc=%4.4d-%2.2d-%2.2d+%2.2d%%3a%2.2d%%3a%2.2d",
                       gmTime.tm_year + 1900, gmTime.tm_mon + 1, gmTime.tm_mday, 
                       gmTime.tm_hour, gmTime.tm_min, gmTime.tm_sec);

    // check for any wind registered
    if (notify->wspeed >= 0 || notify->hiwspeed >= 0)
    {
        length += sprintf (&httpBuffer[length], "&winddir=%3.3d", notify->winddir);
    }
    
    length += sprintf (&httpBuffer[length], "&windspeedmph=%d.0", 
                       (notify->wspeed >= 0) ? notify->wspeed : 0);
    length += sprintf (&httpBuffer[length], "&windgustmph=%d.0", 
                       (notify->hiwspeed >= 0) ? notify->hiwspeed : 0);

    length += sprintf (&httpBuffer[length], "&tempf=%d.%d", 
                       notify->temp/10, notify->temp%10);
    
    length += sprintf (&httpBuffer[length], "&rainin=%.2f", rainIN);
    
    length += sprintf (&httpBuffer[length], "&baromin=%2.2d.%2.2d", 
                       notify->barom/1000, (notify->barom%1000)/10);

    length += sprintf (&httpBuffer[length], "&dewptf=%d.%d", 
                       notify->dewpoint/10, notify->dewpoint%10);

    if (notify->humidity >= 0 && notify->humidity <= 100)
    {
        length += sprintf (&httpBuffer[length], "&humidity=%d", notify->humidity);
    }

    strcpy (version, globalWviewVersionStr);
    version[5] = '-';
    length += sprintf (&httpBuffer[length], "&weather=&softwaretype=%s&action=updateraw",
                       version);


    strncpy (tempBfr, httpBuffer, 192);
    wvutilsLogEvent (PRI_STATUS, "WEATHERFORYOU-send: %s", tempBfr);
    if (strlen(httpBuffer) > 192)
    {
        strncpy (tempBfr, &httpBuffer[192], 192);
        wvutilsLogEvent (PRI_STATUS, "WEATHERFORYOU-send: %s", tempBfr);
    }


    // OK, now use libcurl to execute the HTTP "GET"
    curl = curl_easy_init ();
    if (curl == NULL)
    {
        radMsgLog (PRI_HIGH, "WEATHERFORYOU-error: failed to initialize curl!");
        return;
    }
    
    curl_easy_setopt (curl, CURLOPT_URL, httpBuffer);
    curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, recv_data);
    curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, curlError);
    curl_easy_setopt (curl, CURLOPT_TIMEOUT, 60);

    res = curl_easy_perform (curl);
    if (res != 0)
    {
        radMsgLog (PRI_HIGH, "WEATHERFORYOU-error: %s", curlError);
    }
    
    curl_easy_cleanup (curl);
    return;
}

static int waitForWviewDaemon (void)
{
    WVIEW_MSG_REQUEST       msg;
    char                    srcQName[QUEUE_NAME_LENGTH+1];
    UINT                    msgType;
    UINT                    length;
    void                    *recvBfr;
    int                     retVal, done = FALSE;
    WVIEW_MSG_ARCHIVE_PATH  *apMsg;

    // enable message reception from the radlib router for the archive path
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon: radMsgRouterMessageSend failed!");
        radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
        return ERROR;
    }

    // now wait for the response here
    while (!done)
    {
        radUtilsSleep (50);
        
        
        if ((retVal = radQueueRecv (radProcessQueueGetID (),
                                    srcQName,
                                    &msgType,
                                    &recvBfr,
                                    &length))
            == FALSE)
        {
            continue;
        }
        else if (retVal == ERROR)
        {
            radMsgLog (PRI_STATUS, "waitForWviewDaemon: queue is closed!");
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            return ERROR;
        }
    
        // is this what we want?
        if (msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            // yes!
            done = TRUE;
            apMsg = (WVIEW_MSG_ARCHIVE_PATH*)recvBfr;
            strncpy (httpWork.archivePath, apMsg->archivePath, 256);
        }
    
        // release the received buffer
        radBufferRls (recvBfr);
        }
    
    // disable message reception from the radlib router for the archive path
    radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    return OK;
}

/*  ... system initialization
*/
static int httpSysInit (WVIEW_HTTPD_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, HTTP_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_HTTP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_HTTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, HTTP_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_HTTP);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_HTTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int httpSysExit (WVIEW_HTTPD_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}

static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wvhttpd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGILL:
        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wvhttpd: recv sig %d: bailing out!", signum);
            abort ();
        
        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            if (httpWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }

            // We exit now, we are probably hung on a "curl_easy_perform" call!
            radMsgLog (PRI_HIGH, "wvhttpd: recv sig %d: exiting now!", signum);
            radMsgRouterExit ();
            httpSysExit (&httpWork);
            radProcessExit ();
            radSystemExit (WVIEW_SYSTEM_ID);
            exit (0);                
    }
    
    return;
}

static void msgHandler
(
    char                        *srcQueueName,
    UINT                        msgType,
    void                        *msg,
    UINT                        length,
    void                        *userData
)
{
    WVIEW_MSG_ARCHIVE_NOTIFY    *anMsg;
    time_t                      nowTime = time (NULL);
    
    if (msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
    {
        anMsg = (WVIEW_MSG_ARCHIVE_NOTIFY *)msg;

        // Update the rain accumulator
        sensorAccumAddSample (httpWork.rainAccumulator, nowTime, anMsg->sampleRain);

        // process http data transfer
        if (strcmp (httpWork.stationId, "0"))
        {
            processWUNDERGROUND (anMsg);
        }
        if (strcmp (httpWork.youstationId, "0"))
        {
            processWEATHERFORYOU (anMsg);
        }
    }
    else if (msgType == WVIEW_MSG_TYPE_POLL)
    {
        WVIEW_MSG_POLL*     pPoll = (WVIEW_MSG_POLL*)msg;
        wvutilsSendPMONPollResponse (pPoll->mask, PMON_PROCESS_WVHTTPD);
        return;
    }

    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    return;
}

static void timerHandler (void *parm)
{
    return;
}

/*  ... the main entry point for the wvhttpd process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             retVal;
    FILE            *pidfile;
    char            instance[32], value[_MAX_PATH];
    CF_ID           configFileId;
    struct stat     fileStatus;
    time_t          timeStamp, nowTime = time (NULL) - WV_SECONDS_IN_HOUR;
    struct tm       buildTime, *bknTime = localtime (&nowTime);
    USHORT          recdate, rectime;
    ArchiveRecord   recordStore;
    float           tempRain;

    memset (&httpWork, 0, sizeof (httpWork));

    /*  ... initialize some system stuff first
    */
    retVal = httpSysInit (&httpWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_HTTP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wvhttpd init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_HTTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_HTTP,
                        httpWork.fifoFile,
                        PROC_NUM_TIMERS_HTTP,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_HTTP);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    httpWork.myPid = getpid ();
    pidfile = fopen (httpWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVWUNDERD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!");
        httpSysExit (&httpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    /*  ... get our configuration values
    */
    if (stat (httpWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...", 
                   httpWork.configFile);
        httpSysExit (&httpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (httpWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen failed!");
        httpSysExit (&httpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // get the STATION ID
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATIONID],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s not found: WUNDERGROUND set to disabled", configIDs[CFG_ID_STATIONID]);
        strcpy (httpWork.stationId, "0");
    }
    else
    {
        strcpy (httpWork.stationId, value);
        strcpy (httpWork.password, "0");
    }

    // get the password
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_PASSWORD],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s not found: WUNDERGROUND set to disabled", configIDs[CFG_ID_PASSWORD]);
        strcpy (httpWork.stationId, "0");
        strcpy (httpWork.password, "0");
    }
    else
    {
        strcpy (httpWork.password, value);
    }

    // get the YOU STATION ID
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_YOUSTATIONID],
                            instance,
                            value)
        == ERROR)
    {
        // default to disabled
        radMsgLog (PRI_CATASTROPHIC, "%s not found: WEATHERFORYOU set to disabled", configIDs[CFG_ID_YOUSTATIONID]);
        strcpy (httpWork.youstationId, "0");
    }
    else
    {
        strcpy (httpWork.youstationId, value);
        strcpy (httpWork.youpassword, "0");        
    }

    // get the YOU password
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_YOUPASSWORD],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s not found: WEATHERFORYOU set to disabled", configIDs[CFG_ID_YOUPASSWORD]);
        strcpy (httpWork.youstationId, "0");
        strcpy (httpWork.youpassword, "0");        
    }
    else
    {
        strcpy (httpWork.youpassword, value);
    }

    radCfClose (configFileId);

    if (strcmp (httpWork.stationId, "0"))
    {
        radMsgLog (PRI_STATUS, "WUNDERGROUND: configured to submit station %s data to wunderground.com",
                   httpWork.stationId);
    }

    if (strcmp (httpWork.youstationId, "0"))
    {
        radMsgLog (PRI_STATUS, "WEATHERFORYOU: configured to submit station %s data to weatherforyou.com",
                   httpWork.youstationId);
    }

    // wait a bit here before continuing
    radUtilsSleep (500);


    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        httpSysExit (&httpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // wait for the wview daemon to be ready
    if (waitForWviewDaemon () == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon failed");
        httpSysExit (&httpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // Create the rain accumulator
    httpWork.rainAccumulator = sensorAccumInit (60);

    // Populate the accumulator
    recdate = INSERT_PACKED_DATE(bknTime->tm_year+1900,bknTime->tm_mon+1,bknTime->tm_mday);
    rectime = (bknTime->tm_hour * 100) + bknTime->tm_min;
    while (dbfGetNextArchiveRecord (httpWork.archivePath, &recdate, &rectime, &recordStore)
           != ERROR)
    {
        memset (&buildTime, 0, sizeof(buildTime));
        buildTime.tm_min    = EXTRACT_PACKED_MINUTE(rectime);
        buildTime.tm_hour   = EXTRACT_PACKED_HOUR(rectime);
        buildTime.tm_mday   = EXTRACT_PACKED_DAY(recdate);
        buildTime.tm_mon    = EXTRACT_PACKED_MONTH(recdate) - 1;
        buildTime.tm_year   = EXTRACT_PACKED_YEAR(recdate) - 1900;
        timeStamp = mktime(&buildTime);
        tempRain = (float)(recordStore.rain & 0xFFF);
        tempRain /= 100.0;
        sensorAccumAddSample (httpWork.rainAccumulator, timeStamp, tempRain);
    }

    // enable message reception from the radlib router for archive notifications
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY);

    // enable message reception from the radlib router for POLL msgs
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_POLL);


    // enter normal processing
    httpWork.inMainLoop = TRUE;
    radMsgLog (PRI_STATUS, "running...");
    
    while (!httpWork.exiting)
    {
        // wait on something interesting
        if (radProcessWait (0) == ERROR)
        {
            httpWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    sensorAccumExit (httpWork.rainAccumulator);
    radMsgRouterExit ();
    httpSysExit (&httpWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

